@@ -118,14 +118,15 @@ soup_add_idle (GMainContext *async_context,
 }
 
 GSource *
-soup_add_completion_reffed (GMainContext *async_context,
-			    GSourceFunc   function,
-			    gpointer      data)
+soup_add_completion_reffed (GMainContext   *async_context,
+			    GSourceFunc     function,
+			    gpointer        data,
+			    GDestroyNotify  dnotify)
 {
 	GSource *source = g_idle_source_new ();
 
 	g_source_set_priority (source, G_PRIORITY_DEFAULT);
-	g_source_set_callback (source, function, data, NULL);
+	g_source_set_callback (source, function, data, dnotify);
 	g_source_attach (source, async_context);
 	return source;
 }
@@ -152,7 +153,7 @@ soup_add_completion (GMainContext *async_context,
 {
 	GSource *source;
 
-	source = soup_add_completion_reffed (async_context, function, data);
+	source = soup_add_completion_reffed (async_context, function, data, NULL);
 	g_source_unref (source);
 	return source;
 }
@@ -256,9 +257,6 @@ const char soup_char_attributes[] = {
 gboolean
 soup_host_matches_host (const gchar *host, const gchar *compare_with)
 {
-	char *match;
-	int dlen;
-
 	g_return_val_if_fail (host != NULL, FALSE);
 	g_return_val_if_fail (compare_with != NULL, FALSE);
 
@@ -268,11 +266,5 @@ soup_host_matches_host (const gchar *host, const gchar *compare_with)
 		return FALSE;
 	if (!g_ascii_strcasecmp (host + 1, compare_with))
 		return TRUE;
-	dlen = strlen (host);
-	while ((match = strstr (compare_with, host))) {
-		if (!match[dlen])
-			return TRUE;
-		compare_with = match + 1;
-	}
-	return FALSE;
+	return g_str_has_suffix (compare_with, host);
 }
